#include <stdio.h>
#include <iostream>
#include <windows.h>
#include <mmsystem.h>

using namespace std;
#pragma comment (lib , "winmm.lib")

#include <irrlicht.h>
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
#pragma comment(lib, "Irrlicht.lib")

#include <Cg/cg.h>
#include "IrrCg.h"
using namespace IrrCg;

const s32 winWidth = 1244;
const s32 winHeight = 700;
IrrlichtDevice* device = 0;
IVideoDriver* driver = 0;
ICameraSceneNode* camera = 0;

class ShaderCallBack : public ICgShaderConstantSetCallBack {
public:
	CGparameter	WorldViewProjection;
	CGparameter	WorldInverseTrans;
	CGparameter	TimerPixel;
	CGparameter	TimerVertex;
	virtual void OnSetConstants(ICgServices* services, CGprogram Vertex, CGprogram Pixel, const SMaterial &Material) {
		WorldViewProjection = cgGetNamedParameter(Vertex, "WorldViewProj");
		matrix4 worldViewProj;
		worldViewProj = driver->getTransform(ETS_PROJECTION);
		worldViewProj *= driver->getTransform(ETS_VIEW);
		worldViewProj *= driver->getTransform(ETS_WORLD);
		services->setMatrix(WorldViewProjection, ICGT_MATRIX_IDENTITY, worldViewProj);
		matrix4 World = driver->getTransform(ETS_WORLD);
		matrix4 WorldInverseTranspose;
		World.getInverse(WorldInverseTranspose);
		WorldInverseTranspose.getTransposed(WorldInverseTranspose);
		WorldInverseTrans = cgGetNamedParameter(Vertex, "WorldInverseTranspose");
		services->setMatrix(WorldInverseTrans, ICGT_MATRIX_IDENTITY, WorldInverseTranspose);
		float time = (float)device->getTimer()->getRealTime();
		TimerPixel = cgGetNamedParameter(Pixel, "Timer");
		TimerVertex = cgGetNamedParameter(Vertex, "Timer");
		services->setParameter3f(TimerPixel, time, 1.0f, 1.0f);
		services->setParameter3f(TimerVertex, time, 1.0f, 1.0f);
	}
};

class InputEventReceiver : public IEventReceiver {
public:
	virtual bool OnEvent(const SEvent& event) {
		if (event.EventType == EET_KEY_INPUT_EVENT)
			KeyIsDown[event.KeyInput.Key] = event.KeyInput.PressedDown;
		if(event.EventType == EET_KEY_INPUT_EVENT && event.KeyInput.Key == KEY_ESCAPE && !event.KeyInput.PressedDown) {
			device->closeDevice();
			return true;
		}
		if(event.EventType == EET_KEY_INPUT_EVENT && event.KeyInput.Key == KEY_KEY_Q && !event.KeyInput.PressedDown) {
			device->closeDevice();
			return true;
		}
		return false;
	}
	virtual bool IsKeyDown(EKEY_CODE keyCode) const {
		return KeyIsDown[keyCode];
	}
	InputEventReceiver() {
		for (u32 i = 0; i < KEY_KEY_CODES_COUNT; ++i) KeyIsDown[i] = false;
	}
private:
	bool KeyIsDown[KEY_KEY_CODES_COUNT];
};

int main(int argc, char** argv) {
	InputEventReceiver receiver;
	SIrrlichtCreationParameters param;
	param.AntiAlias = 0;
	param.Bits = 32;
	param.DriverType = EDT_DIRECT3D9;
	param.EventReceiver = &receiver;
	if (argc > 1) {
		IrrlichtDevice *nullDevice = createDevice(EDT_NULL);
		dimension2d<u32> res = nullDevice->getVideoModeList()->getDesktopResolution();
		nullDevice->drop();
		param.WindowSize.Width = res.Width;
		param.WindowSize.Height = res.Height;
		param.Fullscreen = true;
	} else {
		param.WindowSize.Width = winWidth;
		param.WindowSize.Height = winHeight;
		param.Fullscreen = false;
	}
	param.HighPrecisionFPU = true;
	param.Stencilbuffer = true;
	param.Vsync = false;
	device = createDeviceEx(param);
	driver = device->getVideoDriver();
	ISceneManager* sceneManager = device->getSceneManager();
	
	SKeyMap keyMap[8];
	keyMap[0].Action = EKA_MOVE_FORWARD;
	keyMap[0].KeyCode = KEY_UP;
	keyMap[1].Action = EKA_MOVE_FORWARD;
	keyMap[1].KeyCode = KEY_KEY_W;
	keyMap[2].Action = EKA_MOVE_BACKWARD;
	keyMap[2].KeyCode = KEY_DOWN;
	keyMap[3].Action = EKA_MOVE_BACKWARD;
	keyMap[3].KeyCode = KEY_KEY_S;
	keyMap[4].Action = EKA_STRAFE_LEFT;
	keyMap[4].KeyCode = KEY_LEFT;
	keyMap[5].Action = EKA_STRAFE_LEFT;
	keyMap[5].KeyCode = KEY_KEY_A;
	keyMap[6].Action = EKA_STRAFE_RIGHT;
	keyMap[6].KeyCode = KEY_RIGHT;
	keyMap[7].Action = EKA_STRAFE_RIGHT;
	keyMap[7].KeyCode = KEY_KEY_D;
	camera = sceneManager->addCameraSceneNodeFPS(0, 40.0f, 0.4f, -1, keyMap, 8, false);
	camera->setPosition(vector3df(0.0f, 120.0f, 0.0f));
	vector3df floor = vector3df(0.0f, 10.0f, 0.0f);
	ICgProgrammingServices* gpu = new ICgProgrammingServices(device);

	// palmLeafsBark
	c8* palmLeafsBarkFileName = "shaders/palmLeafsBark.cg"; s32 palmLeafsBark = -1;
	ShaderCallBack* shaderCallBackpalmLeafsBark = new ShaderCallBack();
	palmLeafsBark = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,palmLeafsBarkFileName,"main_v","arbvp1","vs_2_0",palmLeafsBarkFileName,"main_f","arbfp1","ps_2_0",shaderCallBackpalmLeafsBark,EMT_SOLID);
	IAnimatedMesh* palmLeafsBarkMesh = sceneManager->getMesh("obj/palmLeafsBark.obj");
	ISceneNode* palmLeafsBarkSceneNode = 0;
	IMesh* palmLeafsBarkTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(palmLeafsBarkMesh->getMesh(0));
	palmLeafsBarkSceneNode= sceneManager->addMeshSceneNode(palmLeafsBarkTangentMesh);
	palmLeafsBarkTangentMesh->drop();
	// palmLeafsBark Material
	palmLeafsBarkSceneNode->setMaterialType((E_MATERIAL_TYPE)palmLeafsBark);
	palmLeafsBarkSceneNode->getMaterial(0).Lighting = false;
	// palmLeafsBark Maps
	ITexture* palmLeafsBarkMap1 = driver->getTexture("maps/palmLeafsBark.png");
	palmLeafsBarkSceneNode->setMaterialTexture(0, palmLeafsBarkMap1);
	ITexture* palmLeafsBarkMap2 = driver->getTexture("maps/white.png");
	palmLeafsBarkSceneNode->setMaterialTexture(1, palmLeafsBarkMap2);
	ITexture* palmLeafsBarkMap3 = driver->getTexture("maps/palmLeafsBarkNormal.png");
	driver->makeNormalMapTexture(palmLeafsBarkMap3, 9.0f);
	palmLeafsBarkSceneNode->setMaterialTexture(2, palmLeafsBarkMap3);

	// Rox
	c8* roxFileName = "shaders/rox.cg"; s32 rox = -1;
	ShaderCallBack* shaderCallBackRox = new ShaderCallBack();
	rox = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,roxFileName,"main_v","arbvp1","vs_2_0",roxFileName,"main_f","arbfp1","ps_2_0",shaderCallBackRox,EMT_SOLID);
	IAnimatedMesh* roxMesh = sceneManager->getMesh("obj/rox.obj");
	ISceneNode* roxSceneNode = 0;
	IMesh* roxTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(roxMesh->getMesh(0));
	roxSceneNode= sceneManager->addMeshSceneNode(roxTangentMesh);
	roxTangentMesh->drop();
	// Rox Material
	roxSceneNode->setMaterialType((E_MATERIAL_TYPE)rox);
	roxSceneNode->getMaterial(0).Lighting = false;
	// Rox Maps
	ITexture* roxMap1 = driver->getTexture("maps/roxDiffuse.png");
	roxSceneNode->setMaterialTexture(0, roxMap1);
	ITexture* roxMap2 = driver->getTexture("maps/white.png");
	roxSceneNode->setMaterialTexture(1, roxMap2);
	ITexture* roxMap3 = driver->getTexture("maps/roxNormal.png");
	driver->makeNormalMapTexture(roxMap3, 9.0f);
	roxSceneNode->setMaterialTexture(2, roxMap3);

	// roxSmall
	c8* roxSmallFileName = "shaders/rox.cg"; s32 roxSmall = -1;
	ShaderCallBack* shaderCallBackroxSmall = new ShaderCallBack();
	roxSmall = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,roxSmallFileName,"main_v","arbvp1","vs_2_0",roxSmallFileName,"main_f","arbfp1","ps_2_0",shaderCallBackroxSmall,EMT_SOLID);
	IAnimatedMesh* roxSmallMesh = sceneManager->getMesh("obj/roxSmall.obj");
	ISceneNode* roxSmallSceneNode = 0;
	IMesh* roxSmallTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(roxSmallMesh->getMesh(0));
	roxSmallSceneNode= sceneManager->addMeshSceneNode(roxSmallTangentMesh);
	roxSmallTangentMesh->drop();
	// roxSmall Material
	roxSmallSceneNode->setMaterialType((E_MATERIAL_TYPE)roxSmall);
	roxSmallSceneNode->getMaterial(0).Lighting = false;
	// roxSmall Maps
	ITexture* roxSmallMap1 = driver->getTexture("maps/roxDiffuse.png");
	roxSmallSceneNode->setMaterialTexture(0, roxSmallMap1);
	ITexture* roxSmallMap2 = driver->getTexture("maps/white.png");
	roxSmallSceneNode->setMaterialTexture(1, roxSmallMap2);
	ITexture* roxSmallMap3 = driver->getTexture("maps/roxNormal.png");
	driver->makeNormalMapTexture(roxSmallMap3, 9.0f);
	roxSmallSceneNode->setMaterialTexture(2, roxSmallMap3);

	// Water
	c8* waterFileName = "shaders/water.cg"; s32 water = -1;
	ShaderCallBack* waterCallBack = new ShaderCallBack();
	water = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,waterFileName,"main_v","arbvp1","vs_3_0",waterFileName,"main_f","arbfp1","ps_3_0",waterCallBack,EMT_TRANSPARENT_ALPHA_CHANNEL);
	IAnimatedMesh* waterMesh = sceneManager->getMesh("obj/water.obj");
	ISceneNode* waterSceneNode = 0;
	IMesh* waterTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(waterMesh->getMesh(0));
	waterSceneNode = sceneManager->addMeshSceneNode(waterTangentMesh);
	waterTangentMesh->drop();
	// Water Material
	waterSceneNode->setMaterialType((E_MATERIAL_TYPE)water);
	waterSceneNode->getMaterial(0).Lighting = false;
	waterSceneNode->getMaterial(0).BlendOperation = EBO_ADD;
	waterSceneNode->getMaterial(0).MaterialTypeParam = pack_textureBlendFunc(EBF_SRC_ALPHA, EBF_ONE_MINUS_SRC_ALPHA, EMFN_MODULATE_1X, EAS_TEXTURE | EAS_VERTEX_COLOR);
	waterSceneNode->setMaterialFlag(EMF_BLEND_OPERATION, true);
	waterSceneNode->setMaterialFlag(EMF_COLOR_MATERIAL, true);
	// Water Maps
	ITexture* waterMap1 = driver->getTexture("maps/waterSand.png");
	waterSceneNode->setMaterialTexture(0, waterMap1);
	ITexture* waterMap2 = driver->getTexture("maps/waterCaustics.png");
	waterSceneNode->setMaterialTexture(1, waterMap2);
	ITexture* waterMap3 = driver->getTexture("maps/waterFoam.png");
	waterSceneNode->setMaterialTexture(2, waterMap3);
	ITexture* waterMap4 = driver->getTexture("maps/waterMap.png");
	waterSceneNode->setMaterialTexture(3, waterMap4);
	ITexture* waterMap5 = driver->getTexture("maps/waterNormal.png");
	driver->makeNormalMapTexture(waterMap5, 9.0f);
	waterSceneNode->setMaterialTexture(4, waterMap5);
	
	// palmLeafs
	c8* palmLeafsFileName = "shaders/palmLeafs.cg"; s32 palmLeafs = -1;
	ShaderCallBack* shaderCallBackpalmLeafs = new ShaderCallBack();
	palmLeafs = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,palmLeafsFileName,"main_v","arbvp1","vs_3_0",palmLeafsFileName,"main_f","arbfp1","ps_3_0",shaderCallBackpalmLeafs,EMT_SOLID);
	IAnimatedMesh* palmLeafsMesh = sceneManager->getMesh("obj/palmLeafs.obj");
	ISceneNode* palmLeafsSceneNode = 0;
	IMesh* palmLeafsTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(palmLeafsMesh->getMesh(0));
	palmLeafsSceneNode= sceneManager->addMeshSceneNode(palmLeafsTangentMesh);
	palmLeafsTangentMesh->drop();
	// palmLeafs Material
	palmLeafsSceneNode->setMaterialType((E_MATERIAL_TYPE)palmLeafs);
	palmLeafsSceneNode->getMaterial(0).Lighting = false;
	palmLeafsSceneNode->getMaterial(0).BlendOperation = EBO_ADD;
	palmLeafsSceneNode->setMaterialFlag(EMF_BACK_FACE_CULLING, false);
	// palmLeafs Maps
	ITexture* palmLeafsMap1 = driver->getTexture("maps/palmLeafs.png");
	palmLeafsSceneNode->setMaterialTexture(0, palmLeafsMap1);
	ITexture* palmLeafsMap2 = driver->getTexture("maps/palmLeafsAlpha.png");
	palmLeafsSceneNode->setMaterialTexture(1, palmLeafsMap2);

	// sand
	c8* sandFileName = "shaders/sand.cg"; s32 sand = -1;
	ShaderCallBack* shaderCallBacksand = new ShaderCallBack();
	sand = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,sandFileName,"main_v","arbvp1","vs_2_0",sandFileName,"main_f","arbfp1","ps_2_0",shaderCallBacksand,EMT_TRANSPARENT_ALPHA_CHANNEL);
	IAnimatedMesh* sandMesh = sceneManager->getMesh("obj/sand.obj");
	ISceneNode* sandSceneNode = 0;
	IMesh* sandTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(sandMesh->getMesh(0));
	sandSceneNode= sceneManager->addMeshSceneNode(sandTangentMesh);
	sandTangentMesh->drop();
	// sand Material
	sandSceneNode->setMaterialType((E_MATERIAL_TYPE)sand);
	sandSceneNode->getMaterial(0).Lighting = false;
	// sand Map
	ITexture* sandMap1 = driver->getTexture("maps/waterSand.png");
	sandSceneNode->setMaterialTexture(0, sandMap1);
	
	// boat
	c8* boatFileName = "shaders/boat.cg"; s32 boat = -1;
	ShaderCallBack* shaderCallBackboat = new ShaderCallBack();
	boat = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,boatFileName,"main_v","arbvp1","vs_2_0",boatFileName,"main_f","arbfp1","ps_2_0",shaderCallBackboat,EMT_SOLID);
	IAnimatedMesh* boatMesh = sceneManager->getMesh("obj/boat.obj");
	ISceneNode* boatSceneNode = 0;
	IMesh* boatTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(boatMesh->getMesh(0));
	boatSceneNode= sceneManager->addMeshSceneNode(boatTangentMesh);
	boatTangentMesh->drop();
	// boat Material
	boatSceneNode->setMaterialType((E_MATERIAL_TYPE)boat);
	boatSceneNode->getMaterial(0).Lighting = false;
	// boat Map
	ITexture* boatMap1 = driver->getTexture("maps/boat.png");
	boatSceneNode->setMaterialTexture(0, boatMap1);

	// grass
	c8* grassFileName = "shaders/grass.cg"; s32 grass = -1;
	ShaderCallBack* shaderCallBackgrass = new ShaderCallBack();
	grass = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,grassFileName,"main_v","arbvp1","vs_2_0",grassFileName,"main_f","arbfp1","ps_2_0",shaderCallBackgrass,EMT_SOLID);
	IAnimatedMesh* grassMesh = sceneManager->getMesh("obj/grass.obj");
	ISceneNode* grassSceneNode = 0;
	IMesh* grassTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(grassMesh->getMesh(0));
	grassSceneNode= sceneManager->addMeshSceneNode(grassTangentMesh);
	grassSceneNode->setPosition(vector3df(0.0f, -5.0f, 0.0f));
	grassTangentMesh->drop();
	// grass Material
	grassSceneNode->setMaterialType((E_MATERIAL_TYPE)grass);
	grassSceneNode->getMaterial(0).Lighting = false;
	// grass Maps
	ITexture* grassMap1 = driver->getTexture("maps/grass.png");
	grassSceneNode->setMaterialTexture(0, grassMap1);
	ITexture* grassMap2 = driver->getTexture("maps/white.png");
	grassSceneNode->setMaterialTexture(1, grassMap2);
	ITexture* grassMap3 = driver->getTexture("maps/grassNormal.png");
	driver->makeNormalMapTexture(grassMap3, 9.0f);
	grassSceneNode->setMaterialTexture(2, grassMap3);

	// green1
	c8* greenLeafsFileName = "shaders/default.cg"; s32 greenLeafs = -1;
	ShaderCallBack* shaderCallBackgreenLeafs = new ShaderCallBack();
	greenLeafs = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,greenLeafsFileName,"main_v","arbvp1","vs_2_0",greenLeafsFileName,"main_f","arbfp1","ps_2_0",shaderCallBackgreenLeafs,EMT_SOLID);
	IAnimatedMesh* greenLeafsMesh = sceneManager->getMesh("obj/green1.obj");
	ISceneNode* greenLeafsSceneNode = 0;
	IMesh* greenLeafsTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(greenLeafsMesh->getMesh(0));
	greenLeafsSceneNode= sceneManager->addMeshSceneNode(greenLeafsTangentMesh);
	greenLeafsTangentMesh->drop();
	// green1 Material
	greenLeafsSceneNode->setMaterialType((E_MATERIAL_TYPE)greenLeafs);
	greenLeafsSceneNode->getMaterial(0).Lighting = false;
	// green1 Map
	ITexture* greenLeafsMap1 = driver->getTexture("maps/green.png");
	greenLeafsSceneNode->setMaterialTexture(0, greenLeafsMap1);

	// green2
	c8* greenLeafs2FileName = "shaders/default.cg"; s32 greenLeafs2 = -1;
	ShaderCallBack* shaderCallBackgreenLeafs2 = new ShaderCallBack();
	greenLeafs2 = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,greenLeafs2FileName,"main_v","arbvp1","vs_2_0",greenLeafs2FileName,"main_f","arbfp1","ps_2_0",shaderCallBackgreenLeafs2,EMT_SOLID);
	IAnimatedMesh* greenLeafs2Mesh = sceneManager->getMesh("obj/green2.obj");
	ISceneNode* greenLeafs2SceneNode = 0;
	IMesh* greenLeafs2TangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(greenLeafs2Mesh->getMesh(0));
	greenLeafs2SceneNode= sceneManager->addMeshSceneNode(greenLeafs2TangentMesh);
	greenLeafs2TangentMesh->drop();
	// green2 Material
	greenLeafs2SceneNode->setMaterialType((E_MATERIAL_TYPE)greenLeafs2);
	greenLeafs2SceneNode->getMaterial(0).Lighting = false;
	// green2 Map
	ITexture* greenLeafs2Map1 = driver->getTexture("maps/green.png");
	greenLeafs2SceneNode->setMaterialTexture(0, greenLeafs2Map1);

	// green3
	c8* greenLeafs3FileName = "shaders/default.cg"; s32 greenLeafs3 = -1;
	ShaderCallBack* shaderCallBackgreenLeafs3 = new ShaderCallBack();
	greenLeafs3 = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,greenLeafs3FileName,"main_v","arbvp1","vs_2_0",greenLeafs3FileName,"main_f","arbfp1","ps_2_0",shaderCallBackgreenLeafs3,EMT_SOLID);
	IAnimatedMesh* greenLeafs3Mesh = sceneManager->getMesh("obj/green3.obj");
	ISceneNode* greenLeafs3SceneNode = 0;
	IMesh* greenLeafs3TangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(greenLeafs3Mesh->getMesh(0));
	greenLeafs3SceneNode= sceneManager->addMeshSceneNode(greenLeafs3TangentMesh);
	greenLeafs3TangentMesh->drop();
	// green3 Material
	greenLeafs3SceneNode->setMaterialType((E_MATERIAL_TYPE)greenLeafs3);
	greenLeafs3SceneNode->getMaterial(0).Lighting = false;
	// green3 Map
	ITexture* greenLeafs3Map1 = driver->getTexture("maps/green.png");
	greenLeafs3SceneNode->setMaterialTexture(0, greenLeafs3Map1);

	// green4
	c8* greenLeafs4FileName = "shaders/default.cg"; s32 greenLeafs4 = -1;
	ShaderCallBack* shaderCallBackgreenLeafs4 = new ShaderCallBack();
	greenLeafs4 = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,greenLeafs4FileName,"main_v","arbvp1","vs_2_0",greenLeafs4FileName,"main_f","arbfp1","ps_2_0",shaderCallBackgreenLeafs4,EMT_SOLID);
	IAnimatedMesh* greenLeafs4Mesh = sceneManager->getMesh("obj/green4.obj");
	ISceneNode* greenLeafs4SceneNode = 0;
	IMesh* greenLeafs4TangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(greenLeafs4Mesh->getMesh(0));
	greenLeafs4SceneNode= sceneManager->addMeshSceneNode(greenLeafs4TangentMesh);
	greenLeafs4TangentMesh->drop();
	// green4 Material
	greenLeafs4SceneNode->setMaterialType((E_MATERIAL_TYPE)greenLeafs4);
	greenLeafs4SceneNode->getMaterial(0).Lighting = false;
	// green4 Map
	ITexture* greenLeafs4Map1 = driver->getTexture("maps/green.png");
	greenLeafs4SceneNode->setMaterialTexture(0, greenLeafs4Map1);

	// treasure
	c8* treasureFileName = "shaders/default.cg"; s32 treasure = -1;
	ShaderCallBack* shaderCallBacktreasure = new ShaderCallBack();
	treasure = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,treasureFileName,"main_v","arbvp1","vs_2_0",treasureFileName,"main_f","arbfp1","ps_2_0",shaderCallBacktreasure,EMT_SOLID);
	IAnimatedMesh* treasureMesh = sceneManager->getMesh("obj/treasure.obj");
	ISceneNode* treasureSceneNode = 0;
	IMesh* treasureTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(treasureMesh->getMesh(0));
	treasureSceneNode= sceneManager->addMeshSceneNode(treasureTangentMesh);
	treasureTangentMesh->drop();
	// treasure Material
	treasureSceneNode->setMaterialType((E_MATERIAL_TYPE)treasure);
	treasureSceneNode->getMaterial(0).Lighting = false;
	// treasure Map
	ITexture* treasureMap1 = driver->getTexture("maps/treasure.png");
	treasureSceneNode->setMaterialTexture(0, treasureMap1);

	// gold
	c8* goldFileName = "shaders/default.cg"; s32 gold = -1;
	ShaderCallBack* shaderCallBackgold = new ShaderCallBack();
	gold = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,goldFileName,"main_v","arbvp1","vs_2_0",goldFileName,"main_f","arbfp1","ps_2_0",shaderCallBackgold,EMT_SOLID);
	IAnimatedMesh* goldMesh = sceneManager->getMesh("obj/gold.obj");
	ISceneNode* goldSceneNode = 0;
	IMesh* goldTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(goldMesh->getMesh(0));
	goldSceneNode= sceneManager->addMeshSceneNode(goldTangentMesh);
	goldTangentMesh->drop();
	// gold Material
	goldSceneNode->setMaterialType((E_MATERIAL_TYPE)gold);
	goldSceneNode->getMaterial(0).Lighting = false;
	// gold Map
	ITexture* goldMap1 = driver->getTexture("maps/gold.png");
	goldSceneNode->setMaterialTexture(0, goldMap1);

	// barrel
	c8* barrelFileName = "shaders/default.cg"; s32 barrel = -1;
	ShaderCallBack* shaderCallBackbarrel = new ShaderCallBack();
	barrel = gpu->addCgShaderMaterialFromFiles(CG_SOURCE,barrelFileName,"main_v","arbvp1","vs_2_0",barrelFileName,"main_f","arbfp1","ps_2_0",shaderCallBackbarrel,EMT_SOLID);
	IAnimatedMesh* barrelMesh = sceneManager->getMesh("obj/barrel.obj");
	ISceneNode* barrelSceneNode = 0;
	IMesh* barrelTangentMesh = sceneManager->getMeshManipulator()->createMeshWithTangents(barrelMesh->getMesh(0));
	barrelSceneNode= sceneManager->addMeshSceneNode(barrelTangentMesh);
	barrelTangentMesh->drop();
	// barrel Material
	barrelSceneNode->setMaterialType((E_MATERIAL_TYPE)barrel);
	barrelSceneNode->getMaterial(0).Lighting = false;
	// barrel Map
	ITexture* barrelMap1 = driver->getTexture("maps/barrel.png");
	barrelSceneNode->setMaterialTexture(0, barrelMap1);

	// skyBox
	ISceneNode* skyBox = sceneManager->addSkyBoxSceneNode(
		driver->getTexture("maps/cubeMapposy.png"),
		driver->getTexture("maps/cubeMapnegy.png"),
		driver->getTexture("maps/cubeMapposz.png"),
		driver->getTexture("maps/cubeMapnegz.png"),
		driver->getTexture("maps/cubeMapposx.png"),
		driver->getTexture("maps/cubeMapnegx.png"));

	device->getCursorControl()->setVisible(false);
	PlaySound(".\\audio\\water.wav", NULL, SND_ASYNC | SND_LOOP | SND_FILENAME);
	printf("\n  Treasure Island\n  Realtime3D Demo DirectX9\n" \
		"\n  http://codewelt.com\n");
	stringw str = "Treasure Island - Realtime3D Demo DirectX9 - http://codewelt.com";
	device->setWindowCaption(str.c_str());
	while(device->run()) {
		if(device->isWindowActive()) {
			driver->beginScene(true, true, SColor(255, 255, 255, 255));
			sceneManager->drawAll();
			driver->endScene();
		}
	}
	delete gpu;
	device->drop();
	return 0;
}